/*-----------------------------------------------------------------------------
FILENAME: timer.h
AUTHOR: S. Seegel, post@seegel-systeme.de
DATE: 22.7.2008
LICENSE: GPL v3
DESCRIPTION: Implementation of tagmanager class
-----------------------------------------------------------------------------*/

#include "tagmanager.h"
#include <avr/eeprom.h>

#include <string.h>
#include "global.h"

Tagmanager tagmanager;

const uint8_t empty_tag[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

Tagmanager::Tagmanager(void)
{
	mode = TMM_IDLE;
	admin = false;
}

void Tagmanager::Init(void)
{
}

void Tagmanager::Execute(void)
{
	uint8_t temp[5];
	
	switch (mode)
	{
		case TMM_LOOKUP:
			eeprom_read_block(temp, eeprom_ptr, 5);
			
			if (memcmp(temp, tag, 5) == 0)
			{
				GiveAccess(); //Tag found
				if (eeprom_ptr == 0) //administrator tag ? (first tag in eeprom)
					admin = true;
			}
			else
				if (memcmp(temp, empty_tag, 5) == 0)
				{//eeprom tag is empty, we are at the end of eeprom-tags
					if (eeprom_ptr == 0)
					{//no tags in eeprom; add tag (this is now the administrator tag)
						eeprom_write_block(tag, 0, 5);
						GiveAccess();
					}
					else
						if (admin)
							AddCurrentTag();
						else
							DenyAccess();
				}
				else
					eeprom_ptr += 5;
			break;

			
		case TMM_IDLE:
			if (timer.IsFlagged())
			{
				admin = false;
				RELAISOFF;
				mode = TMM_IDLE;
				LEDOFF;
			}
			break;
	}
}

void Tagmanager::ProcessTag(uint8_t *tag)
{
	if (mode == TMM_IDLE)
	{
		eeprom_ptr = 0;
		memcpy(this->tag, tag, 5);
		mode = TMM_LOOKUP;
	}
}

void Tagmanager::GiveAccess(void)
{	
	mode = TMM_IDLE;
	timer.SetTime(2000);
	RELAISON;
	LEDGREEN;
}

void Tagmanager::DenyAccess(void)
{
	mode = TMM_IDLE;
	timer.SetTime(500);
	LEDRED;
}

void Tagmanager::AddCurrentTag(void)
{
	mode = TMM_IDLE;
	admin = false;
	eeprom_write_block(tag, eeprom_ptr, 5);
	
}
